@@ -7,7 +7,7 @@
 
 /***********************************************************************
  *
- * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002,2003,2005,2006,2007,2008
+ * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002,2003,2005,2006,2007,2008,2009,2011
  * Free Software Foundation, Inc.
  * Written by Steve Byrne.
  *
@@ -59,6 +59,13 @@
 #include "ffi.h"
 #include <ltdl.h>
 
+#ifdef HAVE_GETGRNAM
+#include <grp.h>
+#endif
+#ifdef HAVE_GETPWNAM
+#include <pwd.h>
+#endif
+
 typedef struct cparam
 {
   union {
@@ -67,6 +74,7 @@ typedef struct cparam
     float floatVal;
     double doubleVal;
     long double longDoubleVal;
+    long long longLongVal;
     struct {
       PTR pPtrVal;
       PTR ptrVal;
@@ -143,9 +151,10 @@ static void marli (int n);
 static void bad_type (OOP class_oop,
 		      cdata_type cType);
 
-/* Determines the appropriate C mapping for OOP and stores it.  */
-static void push_smalltalk_obj (OOP oop,
-				cdata_type cType);
+/* Determines the appropriate C mapping for OOP and stores it. This
+   returns 1 in case the type could not be converted. */
+static mst_Boolean push_smalltalk_obj (OOP oop,
+				       cdata_type cType);
 
 /* Converts the return type as stored in RESULT to an OOP, based
    on the RETURNTYPEOOP that is stored in the descriptor.  #void is
@@ -178,14 +187,18 @@ static int get_errno (void);
 /* Encapsulate binary incompatibilities between various C libraries.  */
 static int my_stat_old (const char *name,
 		        struct gst_stat_struct * out);
-static int my_lstat_old (const char *name,
-		         struct gst_stat_struct * out);
 static int my_stat (const char *name,
 		    OOP out);
+#ifdef HAVE_LSTAT
+static int my_lstat_old (const char *name,
+		         struct gst_stat_struct * out);
 static int my_lstat (const char *name,
 		     OOP out);
+#endif
 static int my_putenv (const char *str);
+static char **get_environ (void);
 static int my_chdir (const char *str);
+static int my_chown (const char *file, const char *owner, const char *group);
 static int my_symlink (const char* oldpath, const char* newpath);
 static char *my_mkdtemp (char* template);
 static int my_mkdir (const char* name, int mode);
@@ -241,6 +254,8 @@ static const char *c_type_name[] = {
   "wchar_t *",			/* CDATA_WSTRING */
   "wchar_t *",			/* CDATA_WSTRING_OUT */
   "char *",			/* CDATA_SYMBOL_OUT */
+  "long long",			/* CDATA_LONGLONG */
+  "unsigned long long", 	/* CDATA_ULONGLONG */
 };
 
 /* The errno on output from a callout */
@@ -269,7 +284,8 @@ get_errno (void)
      the primitive still fails and the file/socket is closed by the
      Smalltalk code.  */
   if (old == ESHUTDOWN || old == ECONNRESET
-      || old == ECONNABORTED || old == ENETRESET)
+      || old == ECONNABORTED || old == ENETRESET
+      || old == EPIPE)
     return 0;
   else
     return (old);
@@ -380,6 +396,12 @@ my_putenv (const char *str)
   return (putenv (clone));
 }
 
+static char **
+get_environ (void)
+{
+  return environ;
+}
+
 
 int
 my_chdir (const char *dir)
@@ -420,16 +442,22 @@ my_opendir (const char *dir)
   return (result);
 }
 
+long long
+test_longlong (long long aVerylongInt)
+{
+  printf ("Getting a long long 0x%llx\n", aVerylongInt);
+  return aVerylongInt;
+}
+
 void
 test_callin (OOP oop, int(*callback)(const char *))
 {
-  OOP o, sel;
+  OOP o;
   double f;
   int i;
   _gst_str_msg_send (oop, "printNl", NULL);
 
   o = _gst_string_to_oop ("abc");
-  sel = _gst_symbol_to_oop ("printNl");
   _gst_str_msg_send (_gst_str_msg_send (o, ",", o, NULL), "printNl",
 		     NULL);
   i = callback ("this is a test");
@@ -476,7 +504,21 @@ dld_open (const char *filename)
   lt_dlhandle handle;
   void (*initModule) (struct VMProxy *);
 
-  handle = lt_dlopenext (filename);
+  /* Not all shared libraries have .xyz extensions! */
+  handle = lt_dlopen (filename);
+  if (!handle)
+    handle = lt_dlopenext (filename);
+#ifdef __APPLE__
+  if (!handle)
+    {
+      /* For some reason, lt_dlopenext on OS X doesn't try ".dylib" as
+         a possible extension, so we're left with trying it here. */
+      char *full_filename;
+      asprintf(&full_filename, "%s.dylib", filename);
+      handle = lt_dlopen (full_filename);
+      free (full_filename);
+    }
+#endif
   if (handle)
     {
       initModule = lt_dlsym (handle, "gst_initModule");
@@ -489,6 +531,52 @@ dld_open (const char *filename)
   return (NULL);
 #endif
 }
+
+struct search_path_stack {
+  char *saved_search_path;
+  struct search_path_stack *next;
+};
+
+struct search_path_stack *search_paths;
+
+mst_Boolean
+_gst_dlopen (const char *path, mst_Boolean module)
+{
+  PTR h = dld_open (path);
+  if (h && !module)
+    _gst_msg_sendf (NULL, "%v %o addLibraryHandle: %C",
+		    _gst_class_name_to_oop ("DLD"), h);
+  return !!h;
+}
+
+void
+_gst_dladdsearchdir (const char *dir)
+{
+  lt_dlinsertsearchdir (lt_dlgetsearchpath (), dir);
+}
+
+void
+_gst_dlpushsearchpath (void)
+{
+  struct search_path_stack *entry = xmalloc (sizeof (struct search_path_stack));
+  const char *path = lt_dlgetsearchpath ();
+  entry->saved_search_path = path ? strdup (path) : NULL;
+  entry->next = search_paths;
+  search_paths = entry;
+}
+
+void
+_gst_dlpopsearchpath (void)
+{
+  struct search_path_stack *path = search_paths;
+  if (!path)
+    return;
+
+  lt_dlsetsearchpath (path->saved_search_path);
+  search_paths = path->next;
+  free (path->saved_search_path);
+  free (path);
+}
 
 
 
@@ -525,6 +613,7 @@ _gst_init_cfuncs (void)
   _gst_define_cfunc ("getArgv", get_argv);
 
   /* Test functions */
+  _gst_define_cfunc ("testLongLong", test_longlong);
   _gst_define_cfunc ("testCallin", test_callin);
   _gst_define_cfunc ("testCString", test_cstring);
   _gst_define_cfunc ("testCObjectPtr", test_cobject_ptr);
@@ -532,7 +621,9 @@ _gst_init_cfuncs (void)
   /* Access to C library */
   _gst_define_cfunc ("system", system);
   _gst_define_cfunc ("getenv", getenv);
+  _gst_define_cfunc ("environ", get_environ);
   _gst_define_cfunc ("putenv", my_putenv);
+  _gst_define_cfunc ("printf", printf);
 
   _gst_define_cfunc ("errno", get_errno);
   _gst_define_cfunc ("strerror", strerror);
@@ -542,6 +633,7 @@ _gst_init_cfuncs (void)
   _gst_define_cfunc ("lstat_obj", my_lstat);
   _gst_define_cfunc ("utime", _gst_set_file_access_times);
   _gst_define_cfunc ("chmod", chmod);
+  _gst_define_cfunc ("chown", my_chown);
 
   _gst_define_cfunc ("opendir", my_opendir);
   _gst_define_cfunc ("closedir", closedir);
@@ -657,6 +749,11 @@ _gst_c_type_size (int type)
     case CDATA_ULONG:
       return sizeof (unsigned long);
 
+    case CDATA_LONGLONG:
+      return sizeof (long long);
+    case CDATA_ULONGLONG:
+      return sizeof (unsigned long long);
+
     case CDATA_FLOAT:
       return sizeof (float);
     case CDATA_DOUBLE:
@@ -712,7 +809,7 @@ _gst_invoke_croutine (OOP cFuncOOP,
   cparam result, *local_arg_vec, *arg;
   void *funcAddr, **p_slot, **ffi_arg_vec;
   OOP *argTypes, oop;
-  int i, si, fixedArgs, totalArgs;
+  int i, si, fixedArgs, totalArgs, filledArgs;
   mst_Boolean haveVariadic, needPostprocessing;
   inc_ptr incPtr;
 
@@ -761,6 +858,11 @@ _gst_invoke_croutine (OOP cFuncOOP,
         case CDATA_WSTRING_OUT:
         case CDATA_STRING_OUT:
         case CDATA_BYTEARRAY_OUT:
+        case CDATA_SYMBOL_OUT:
+        case CDATA_STRING:
+        case CDATA_BYTEARRAY:
+        case CDATA_SYMBOL:
+        case CDATA_WSTRING:
 	  needPostprocessing = true;
 	  /* fall through */
 
@@ -791,13 +893,25 @@ _gst_invoke_croutine (OOP cFuncOOP,
   /* Push the arguments */
   for (si = i = 0; i < fixedArgs; i++)
     {
+      mst_Boolean res;
+
       cType = IS_OOP (argTypes[i]) ? CDATA_COBJECT : TO_INT (argTypes[i]);
-      if (cType == CDATA_SELF || cType == CDATA_SELF_OOP)
-	push_smalltalk_obj (receiver,
-			    cType == CDATA_SELF ? CDATA_UNKNOWN : CDATA_OOP);
-      else if (cType != CDATA_VOID)
+      if (cType == CDATA_VOID)
+        continue;
+
+      else if (cType == CDATA_SELF || cType == CDATA_SELF_OOP)
+	res = push_smalltalk_obj (receiver,
+				  cType == CDATA_SELF ? CDATA_UNKNOWN : CDATA_OOP);
+      else
 	/* Do nothing if it is a void */
-	push_smalltalk_obj (args[si++], cType);
+	res = push_smalltalk_obj (args[si++], cType);
+
+      if (!res)
+        {
+          oop = NULL;
+	  filledArgs = c_func_cur->arg_idx;
+          goto out;
+        }
     }
 
   /* If the previous call was done through the same function descriptor,
@@ -815,6 +929,8 @@ _gst_invoke_croutine (OOP cFuncOOP,
     }
 
   errno = 0;
+  filledArgs = c_func_cur->arg_idx;
+  assert (filledArgs == totalArgs);
   ffi_call (&c_func_cur->cacheCif, FFI_FN (funcAddr), &result.u, ffi_arg_vec);
 
   _gst_set_errno (errno);
@@ -822,33 +938,38 @@ _gst_invoke_croutine (OOP cFuncOOP,
   oop = c_to_smalltalk (&result, receiver, desc->returnTypeOOP);
   INC_ADD_OOP (oop);
 
+ out:
   /* Fixup all returned string variables */
   if (needPostprocessing)
-    for (i = 0, arg = local_arg_vec; i < totalArgs; i++, arg++)
+    for (i = 0, arg = local_arg_vec; i < filledArgs; i++, arg++)
       {
         if (!arg->oop)
 	  continue;
 
-        switch (arg->cType)
+	switch (arg->cType)
           {
           case CDATA_COBJECT_PTR:
-	    set_cobject_value (arg->oop, arg->u.cObjectPtrVal.ptrVal);
-	    continue;
+            if (oop)
+              set_cobject_value (arg->oop, arg->u.cObjectPtrVal.ptrVal);
+            continue;
 
           case CDATA_WSTRING_OUT:
-	    _gst_set_oop_unicode_string (arg->oop, arg->u.ptrVal);
-	    break;
+            if (oop)
+              _gst_set_oop_unicode_string (arg->oop, arg->u.ptrVal);
+            break;
 
           case CDATA_STRING_OUT:
-	    _gst_set_oopstring (arg->oop, arg->u.ptrVal);
-	    break;
+            if (oop)
+              _gst_set_oopstring (arg->oop, arg->u.ptrVal);
+            break;
 
           case CDATA_BYTEARRAY_OUT:
-	    _gst_set_oop_bytes (arg->oop, arg->u.ptrVal);
-	    break;
+            if (oop)
+              _gst_set_oop_bytes (arg->oop, arg->u.ptrVal);
+            break;
 
           default:
-	    break;
+            break;
           }
 
         xfree (arg->u.ptrVal);
@@ -887,6 +1008,9 @@ get_ffi_type (OOP returnTypeOOP)
 #else
       return &ffi_type_sint64;
 #endif
+    case CDATA_LONGLONG:
+    case CDATA_ULONGLONG:
+      return &ffi_type_sint64;
 
     case CDATA_VOID:
     case CDATA_INT:
@@ -955,9 +1079,13 @@ smalltalk_to_c (OOP oop,
     {
       switch (cType)
         {
+       case CDATA_LONGLONG:
+       case CDATA_ULONGLONG:
+         cp->u.longLongVal = to_c_int_64 (oop);
+         return &ffi_type_sint64;
         case CDATA_LONG:
-	case CDATA_ULONG:
-	  cp->u.longVal = TO_C_LONG (oop);
+       case CDATA_ULONG:
+         cp->u.longVal = TO_C_LONG (oop);
 #if LONG_MAX == 2147483647
           return &ffi_type_sint32;
 #else
@@ -1002,9 +1130,13 @@ smalltalk_to_c (OOP oop,
     {
       switch (cType)
         {
+       case CDATA_LONGLONG:
+       case CDATA_ULONGLONG:
+         cp->u.longLongVal = (long long)(oop == _gst_true_oop);
+         return &ffi_type_sint64;
         case CDATA_LONG:
-	case CDATA_ULONG:
-	  cp->u.longVal = (oop == _gst_true_oop);
+       case CDATA_ULONG:
+         cp->u.longVal = (oop == _gst_true_oop);
 #if LONG_MAX == 2147483647
           return &ffi_type_sint32;
 #else
@@ -1097,6 +1229,7 @@ smalltalk_to_c (OOP oop,
     {				/* how to encode nil */
       switch (cType)
 	{
+	case CDATA_COBJECT_PTR:
 	case CDATA_COBJECT:
 	case CDATA_BYTEARRAY:
 	case CDATA_BYTEARRAY_OUT:
@@ -1129,7 +1262,7 @@ smalltalk_to_c (OOP oop,
   return NULL;
 }
 
-void
+mst_Boolean
 push_smalltalk_obj (OOP oop,
 		    cdata_type cType)
 {
@@ -1139,12 +1272,13 @@ push_smalltalk_obj (OOP oop,
       if (OOP_INT_CLASS (oop) != _gst_array_class)
 	{
           bad_type (OOP_INT_CLASS (oop), cType);
-          return;
+          return false;
 	}
 
       cType = (cType == CDATA_VARIADIC) ? CDATA_UNKNOWN : CDATA_OOP;
       for (i = 1; i <= NUM_WORDS (OOP_TO_OBJ (oop)); i++)
-	push_smalltalk_obj (ARRAY_AT (oop, i), cType);
+	if (!push_smalltalk_obj (ARRAY_AT (oop, i), cType))
+	  return false;
     }
   else
     {
@@ -1154,7 +1288,11 @@ push_smalltalk_obj (OOP oop,
 	INC_ADD_OOP (cp->oop);
       if (type)
 	c_func_cur->types[c_func_cur->arg_idx++] = type;
+      else
+	return false;
     }
+
+  return true;
 }
 
 OOP
@@ -1211,6 +1349,14 @@ c_to_smalltalk (cparam *result, OOP receiverOOP, OOP returnTypeOOP)
       resultOOP = FROM_C_ULONG (result->u.longVal);
       break;
 
+    case CDATA_LONGLONG:
+      resultOOP = FROM_C_LONGLONG (result->u.longLongVal);
+      break;
+
+    case CDATA_ULONGLONG:
+      resultOOP = FROM_C_ULONGLONG (result->u.longLongVal);
+      break;
+
     case CDATA_STRING:
     case CDATA_STRING_OUT:
     case CDATA_WSTRING:
@@ -1283,8 +1429,6 @@ bad_type (OOP class_oop,
   else
     _gst_errorf ("Attempt to pass an instance of %O as a %s", class_oop,
 	         c_type_name[cType]);
-
-  _gst_show_backtrace (stderr);
 }
 
 
@@ -1319,11 +1463,10 @@ closure_msg_send (ffi_cif* cif, void* result, void** args, void* userdata)
 
   desc = (gst_c_callable) OOP_TO_OBJ (callbackOOP);
   cType = IS_OOP (desc->returnTypeOOP) ? CDATA_COBJECT : TO_INT (desc->returnTypeOOP);
-  if (cType != CDATA_VOID)
-    {
-      smalltalk_to_c (resultOOP, &cp, cType);
-      memcpy (result, &cp.u, sizeof (ffi_arg));
-    }
+  if (cType != CDATA_VOID && smalltalk_to_c (resultOOP, &cp, cType))
+    memcpy (result, &cp.u, sizeof (ffi_arg));
+  else
+    memset (result, 0, sizeof (ffi_arg));
 }
 
 void
@@ -1381,6 +1524,106 @@ _gst_set_errno(int errnum)
 }
 
 
+int
+my_chown (const char *file, const char *user, const char *group)
+{
+#if defined HAVE_CHOWN && defined HAVE_GETGRNAM && defined HAVE_GETPWNAM
+  static char *save_user, *save_group;
+  static uid_t save_uid;
+  static gid_t save_gid;
+  static int recursive_depth;
+
+  uid_t uid, gid;
+  if (!file && !user && !group)
+    {
+      recursive_depth--;
+      if (recursive_depth == 0)
+	{
+#if defined HAVE_SETGROUPENT && defined HAVE_ENDGRENT
+	  endgrent ();
+#endif
+#if defined HAVE_SETPASSENT && defined HAVE_ENDPWENT
+	  endpwent ();
+#endif
+	}
+
+      free (save_user);
+      free (save_group);
+      save_user = save_group = NULL;
+      return 0;
+    }
+
+  if (!file)
+    {
+      recursive_depth++;
+      if (recursive_depth == 1)
+	{
+#if defined HAVE_SETGROUPENT && defined HAVE_ENDGRENT
+	  setgroupent (1);
+#endif
+#if defined HAVE_SETPASSENT && defined HAVE_ENDPWENT
+	  setpassent (1);
+#endif
+	}
+    }
+
+  if (!user)
+    uid = -1;
+  else if (save_user && !strcmp (save_user, user))
+    uid = save_uid;
+  else
+    {
+      struct passwd *pw;
+      pw = getpwnam (user);
+      if (!pw)
+	{
+	  errno = EINVAL;
+	  return -1;
+	}
+
+      uid = pw->pw_uid;
+      if (recursive_depth)
+	{
+	  if (save_user)
+	    free (save_user);
+          save_user = strdup (user);
+	  save_uid = uid;
+	}
+    }
+
+  if (!group)
+    gid = -1;
+  else if (save_group && !strcmp (save_group, group))
+    gid = save_gid;
+  else
+    {
+      struct group *gr;
+      gr = getgrnam (group);
+      if (!gr)
+	{
+	  errno = EINVAL;
+	  return -1;
+	}
+
+      gid = gr->gr_gid;
+      if (recursive_depth)
+	{
+	  if (save_group)
+	    free (save_group);
+          save_group = strdup (group);
+	  save_gid = gid;
+	}
+    }
+
+  if (!file)
+    return 0;
+  else
+    return chown (file, uid, gid);
+#else
+  return 0;
+#endif
+}
+
 /* TODO: check if this can be changed to an extern declaration and/or
    an AC_CHECK_DECLS test.  */
 
