@@ -7,7 +7,7 @@
 
 /***********************************************************************
  *
- * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002,2003,2005,2006,2007,2008
+ * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002,2003,2005,2006,2007,2008,2009
  * Free Software Foundation, Inc.
  * Written by Steve Byrne.
  *
@@ -451,7 +451,7 @@ _gst_install_initial_methods (void)
   _gst_set_compilation_category (_gst_string_new ("private"));
   _gst_alloc_bytecodes ();
   _gst_compile_byte (EXIT_INTERPRETER, 0);
-  _gst_compile_byte (RETURN_CONTEXT_STACK_TOP, 0);
+  _gst_compile_byte (JUMP_BACK, 4);
 
   /* The zeros are primitive, # of args, # of temps, stack depth */
   termination_method = _gst_make_new_method (0, 0, 0, 0, _gst_nil_oop,
@@ -465,12 +465,12 @@ _gst_install_initial_methods (void)
 
   install_method (termination_method);
 
-  methodsForString = "\
-methodsFor: aCategoryString [\
-    \"Calling this method prepares the parser to receive methods \
-      to be compiled and installed in the receiver's method dictionary. \
-      The methods are put in the category identified by the parameter.\" \
-    <primitive: VMpr_Behavior_methodsFor> \
+  methodsForString = "\n\
+methodsFor: aCategoryString [\n\
+    \"Calling this method prepares the parser to receive methods \n\
+      to be compiled and installed in the receiver's method dictionary. \n\
+      The methods are put in the category identified by the parameter.\" \n\
+    <primitive: VMpr_Behavior_methodsFor> \n\
 ]";
   _gst_set_compilation_class (_gst_behavior_class);
   _gst_set_compilation_category (_gst_string_new ("compiling methods"));
@@ -674,10 +674,11 @@ _gst_execute_statements (tree_node temps,
   incPtr = INC_SAVE_POINTER ();
   INC_ADD_OOP (methodOOP);
 
-  _gst_bytecode_counter = _gst_primitives_executed =
-    _gst_self_returns = _gst_inst_var_returns = _gst_literal_returns =
-    0;
-  _gst_sample_counter = 0;
+  if (!_gst_raw_profile)
+    _gst_bytecode_counter = _gst_primitives_executed =
+      _gst_self_returns = _gst_inst_var_returns = _gst_literal_returns =
+      _gst_sample_counter = 0;
+
   startTime = _gst_get_milli_time ();
 #ifdef HAVE_GETRUSAGE
   getrusage (RUSAGE_SELF, &startRusage);
@@ -695,81 +696,87 @@ _gst_execute_statements (tree_node temps,
   getrusage (RUSAGE_SELF, &endRusage);
 #endif
 
-  _gst_invoke_hook (GST_AFTER_EVAL);
+  if (!quiet && _gst_verbosity >= 3)
+    {
+      deltaTime = endTime - startTime;
+#ifdef ENABLE_JIT_TRANSLATION
+      printf ("Execution took %.3f seconds", deltaTime / 1000.0);
+#else
+      printf ("%lu byte codes executed\nwhich took %.3f seconds",
+              _gst_bytecode_counter, deltaTime / 1000.0);
+#endif
 
-  INC_RESTORE_POINTER (incPtr);
+#ifdef HAVE_GETRUSAGE
+      deltaTime = ((endRusage.ru_utime.tv_sec * 1000) +
+                   (endRusage.ru_utime.tv_usec / 1000)) -
+        ((startRusage.ru_utime.tv_sec * 1000) +
+         (startRusage.ru_utime.tv_usec / 1000));
+      printf (" (%.3fs user", deltaTime / 1000.0);
+
+      deltaTime = ((endRusage.ru_stime.tv_sec * 1000) +
+                   (endRusage.ru_stime.tv_usec / 1000)) -
+        ((startRusage.ru_stime.tv_sec * 1000) +
+         (startRusage.ru_stime.tv_usec / 1000));
+      printf ("+%.3fs sys)", deltaTime / 1000.0);
+#endif
+      printf ("\n");
+
+#ifndef ENABLE_JIT_TRANSLATION
+      if (_gst_bytecode_counter)
+        {
+          printf ("%lu primitives, percent %.2f\n", _gst_primitives_executed,
+                  100.0 * _gst_primitives_executed / _gst_bytecode_counter);
+          printf ("self returns %lu, inst var returns %lu, literal returns %lu\n",
+                  _gst_self_returns, _gst_inst_var_returns, _gst_literal_returns);
+          printf ("%lu method cache lookups since last cleanup, percent %.2f\n",
+                  _gst_sample_counter,
+                  100.0 * _gst_sample_counter / _gst_bytecode_counter);
+        }
+#endif
+
+      if (_gst_sample_counter)
+        {
+#ifdef ENABLE_JIT_TRANSLATION
+          printf
+            ("%lu primitives, %lu inline cache misses since last cache cleanup\n",
+             _gst_primitives_executed, _gst_sample_counter);
+#endif
+          cacheHits = _gst_sample_counter - _gst_cache_misses;
+          printf ("%lu method cache hits, %lu misses", cacheHits,
+                  _gst_cache_misses);
+          if (cacheHits || _gst_cache_misses)
+            printf (", %.2f percent hits\n", (100.0 * cacheHits) / _gst_sample_counter);
+          else
+            printf ("\n");
+        }
+
+      /* Do more frequent flushing to ensure the result are well placed */
+      printf ("returned value is ");
+      fflush(stdout);
+    }
 
   if (!quiet)
     {
       int save_execution;
 
-      /* Do more frequent flushing to ensure the result are well placed */
-      if (_gst_verbosity >= 3)
-	{
-          printf ("returned value is ");
-          fflush(stdout);
-	}
-
       save_execution = _gst_execution_tracing;
       if (_gst_execution_tracing == 1)
         _gst_execution_tracing = 0;
-      _gst_str_msg_send (_gst_last_returned_value, "printNl", NULL);
+      if (_gst_responds_to (_gst_last_returned_value,
+			    _gst_intern_string ("printNl"))
+          || _gst_responds_to (_gst_last_returned_value,
+			       _gst_does_not_understand_symbol))
+        _gst_str_msg_send (_gst_last_returned_value, "printNl", NULL);
+      else
+	printf ("%O\n", _gst_last_returned_value);
+
+      fflush (stdout);
+      fflush (stderr);
       _gst_execution_tracing = save_execution;
     }
-  if (quiet || _gst_verbosity < 3)
-    return (_gst_last_returned_value);
-
-  deltaTime = endTime - startTime;
-#ifdef ENABLE_JIT_TRANSLATION
-  printf ("Execution took %.3f seconds", deltaTime / 1000.0);
-#else
-  printf ("%lu byte codes executed\nwhich took %.3f seconds",
-	  _gst_bytecode_counter, deltaTime / 1000.0);
-#endif
-
-#ifdef HAVE_GETRUSAGE
-  deltaTime = ((endRusage.ru_utime.tv_sec * 1000) +
-	       (endRusage.ru_utime.tv_usec / 1000)) -
-    ((startRusage.ru_utime.tv_sec * 1000) +
-     (startRusage.ru_utime.tv_usec / 1000));
-  printf (" (%.3fs user", deltaTime / 1000.0);
-
-  deltaTime = ((endRusage.ru_stime.tv_sec * 1000) +
-	       (endRusage.ru_stime.tv_usec / 1000)) -
-    ((startRusage.ru_stime.tv_sec * 1000) +
-     (startRusage.ru_stime.tv_usec / 1000));
-  printf ("+%.3fs sys)", deltaTime / 1000.0);
-#endif
-  printf ("\n");
-
-#ifdef ENABLE_JIT_TRANSLATION
-  if (!_gst_sample_counter)
-    return (_gst_last_returned_value);
-
-  printf
-    ("%lu primitives, %lu inline cache misses since last cache cleanup\n",
-     _gst_primitives_executed, _gst_sample_counter);
-#else
-  if (!_gst_bytecode_counter)
-    return (_gst_last_returned_value);
-
-  printf ("%lu primitives, percent %.2f\n", _gst_primitives_executed,
-	  100.0 * _gst_primitives_executed / _gst_bytecode_counter);
-  printf ("self returns %lu, inst var returns %lu, literal returns %lu\n",
-          _gst_self_returns, _gst_inst_var_returns, _gst_literal_returns);
-  printf ("%lu method cache lookups since last cleanup, percent %.2f\n",
-	  _gst_sample_counter,
-	  100.0 * _gst_sample_counter / _gst_bytecode_counter);
-#endif
-
-  cacheHits = _gst_sample_counter - _gst_cache_misses;
-  printf ("%lu method cache hits, %lu misses", cacheHits,
-	  _gst_cache_misses);
-  if (cacheHits || _gst_cache_misses)
-    printf (", %.2f percent hits\n", (100.0 * cacheHits) / _gst_sample_counter);
-  else
-    printf ("\n");
 
+  _gst_invoke_hook (GST_AFTER_EVAL);
+  INC_RESTORE_POINTER (incPtr);
   return (_gst_last_returned_value);
 }
 
@@ -801,10 +808,17 @@ _gst_compile_method (tree_node method,
   _gst_push_new_scope ();
   inside_block = 0;
   selector = compute_selector (method->v_method.selectorExpr);
-  if (method->location.file_offset != -1 && _gst_get_cur_stream_prompt ())
+
+  /* When we are reading from stdin, it's better to write line numbers where
+     1 is the first line *in the current doit*, because for now the prompt
+     does not include the line number.  This might change in the future.
+
+     Also, do not emit line numbers if the method has no statements.  */
+  if ((method->location.file_offset != -1 && _gst_get_cur_stream_prompt ())
+      || !method->v_method.statements)
     _gst_line_number (method->location.first_line, LN_RESET);
   else
-    _gst_line_number (-1, LN_RESET);
+    _gst_line_number (method->location.first_line, LN_RESET | LN_ABSOLUTE);
 
   INC_ADD_OOP (selector);
 
@@ -1491,12 +1505,12 @@ compile_while_loop (OOP selector,
   for (oldJumpAroundLen = finalJumpLen = 0; ; oldJumpAroundLen = jumpAroundLen)
     {
       finalJumpOfs = whileCondLen + 2 + oldJumpAroundLen + jumpAroundOfs;
-      finalJumpLen = (finalJumpOfs + finalJumpLen > 65536 ? 6 :
-		      finalJumpOfs + finalJumpLen > 256 ? 4 : 2);
-      finalJumpLen = (finalJumpOfs + finalJumpLen > 65536 ? 6 :
-		      finalJumpOfs + finalJumpLen > 256 ? 4 : 2);
-      jumpAroundLen = (jumpAroundOfs + finalJumpLen > 65536 ? 6 :
-		       jumpAroundOfs + finalJumpLen > 256 ? 4 : 2);
+      finalJumpLen = (finalJumpOfs + finalJumpLen >= 65536 ? 6 :
+		      finalJumpOfs + finalJumpLen >= 256 ? 4 : 2);
+      finalJumpLen = (finalJumpOfs + finalJumpLen >= 65536 ? 6 :
+		      finalJumpOfs + finalJumpLen >= 256 ? 4 : 2);
+      jumpAroundLen = (jumpAroundOfs + finalJumpLen >= 65536 ? 6 :
+		       jumpAroundOfs + finalJumpLen >= 256 ? 4 : 2);
       if (jumpAroundLen == oldJumpAroundLen)
         break;
     }
@@ -1543,12 +1557,12 @@ compile_repeat (tree_node receiver)
   repeatedLoopLen = _gst_bytecode_length (receiverExprCodes);
 
   repeatedLoopLen += 2;
-  finalJumpLen = (repeatedLoopLen > 65536 ? 6 :
-		  repeatedLoopLen > 256 ? 4 : 2);
-  finalJumpLen = (repeatedLoopLen + finalJumpLen > 65536 ? 6 :
-		  repeatedLoopLen + finalJumpLen > 256 ? 4 : 2);
-  finalJumpLen = (repeatedLoopLen + finalJumpLen > 65536 ? 6 :
-		  repeatedLoopLen + finalJumpLen > 256 ? 4 : 2);
+  finalJumpLen = (repeatedLoopLen >= 65536 ? 6 :
+		  repeatedLoopLen >= 256 ? 4 : 2);
+  finalJumpLen = (repeatedLoopLen + finalJumpLen >= 65536 ? 6 :
+		  repeatedLoopLen + finalJumpLen >= 256 ? 4 : 2);
+  finalJumpLen = (repeatedLoopLen + finalJumpLen >= 65536 ? 6 :
+		  repeatedLoopLen + finalJumpLen >= 256 ? 4 : 2);
 
   _gst_compile_and_free_bytecodes (receiverExprCodes);
   _gst_compile_byte (POP_STACK_TOP, 0);	/* we don't care about expr's
@@ -2079,7 +2093,6 @@ equal_constant (OOP oop,
 		return (false);
 
 	      varNode = varNode->v_list.next;
-	      pKey = &path->data[i];
 	    }
 	}
       break;
@@ -2223,6 +2236,7 @@ _gst_make_constant_oop (tree_node constExpr)
 	      array->data[i] = _gst_intern_string (varNode->v_list.name);
 	  }
 
+        INC_RESTORE_POINTER (incPtr);
         return (resultOOP);
       }
 
@@ -2836,10 +2850,13 @@ method_new (method_header header,
       else
 	continue;
 
-      MAKE_OOP_UNTRUSTED (blockOOP, IS_OOP_UNTRUSTED (methodOOP));
       block = (gst_compiled_block) OOP_TO_OBJ (blockOOP);
-      block->method = methodOOP;
-      block->literals = literals;
+      if (IS_NIL (block->method))
+	{
+	  MAKE_OOP_UNTRUSTED (blockOOP, IS_OOP_UNTRUSTED (methodOOP));
+	  block->method = methodOOP;
+	  block->literals = literals;
+	}
     }
 
   if (bytecodes != NULL)
