@@ -7,7 +7,7 @@
 
 /***********************************************************************
  *
- * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002,2006,2007,2008
+ * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002,2006,2007,2008,2009
  * Free Software Foundation, Inc.
  * Written by Steve Byrne and Paolo Bonzini.
  *
@@ -95,7 +95,7 @@ int _gst_gc_message = true;
 /* This is != 0 in the middle of a GC.  */
 int _gst_gc_running = 0;
 
-/* These is the memory area which holds the object table.  */
+/* This is the memory area which holds the object table.  */
 static heap oop_heap;
 
 /* This vector holds the storage for all the Character objects in the
@@ -158,15 +158,22 @@ static void grow_memory_no_compact (size_t size);
    memory.  */
 static void reset_survivor_space (struct surv_space *space);
 
+/* Return whether the incremental collector is running.  */
+static inline mst_Boolean incremental_gc_running (void);
+
 /* Restart the incremental collector.  Objects before FIRSTOOP
    are assumed to be alive (currently the base of the OOP table is
    always passed, but you never know).  */
 static void reset_incremental_gc (OOP firstOOP);
 
-/* The incremental collector has done its job.  Update statistics,
-   and if it was also sweeping old objects, make it consider all
-   objects as alive.  */
-static void finished_incremental_gc ();
+/* Compact the old objects.  Grow oldspace to NEWSIZE bytes.  */
+static void compact (size_t new_heap_limit);
+
+/* Allocate and return space for an oldspace object of SIZE bytes.
+   The pointer to the object data is returned, the OOP is
+   stored in P_OOP.  */
+static gst_object alloc_old_obj (size_t size,
+                                 OOP *p_oop);
 
 /* Gather statistics.  */
 static void update_stats (unsigned long *last, double *between, double *duration);
@@ -210,7 +217,7 @@ static int oldspace_sigsegv_handler (void* fault_address, int serious);
 #endif
 
 /* Hook that triggers garbage collection.  */
-static void oldspace_nomemory (heap_data *h, size_t sz);
+static heap_data *oldspace_nomemory (heap_data *h, size_t sz);
 
 /* Answer the number of fields to be scanned in the object starting
    at OBJ, with the given FLAGS on its OOP.  */
@@ -350,16 +357,19 @@ _gst_init_mem (size_t eden, size_t survivor, size_t old,
       _gst_inc_init_registry ();
     }
 
+  _gst_mem.markQueue = (struct mark_queue *)
+    xcalloc (8 * K, sizeof (struct mark_queue));
+  _gst_mem.lastMarkQueue = &_gst_mem.markQueue[8 * K];
 }
 
 void _gst_update_object_memory_oop (OOP oop)
 {
-  int numScavenges;
   gst_object_memory data;
 
   /* Ensure the statistics are coherent.  */
   for (;;) {
-    numScavenges = _gst_mem.numScavenges;
+    OOP floatOOP;
+
     data = (gst_object_memory) OOP_TO_OBJ (oop);
     data->bytesPerOOP = FROM_INT (sizeof (PTR));
     data->bytesPerOTE = FROM_INT (sizeof (struct oop_s) +
@@ -392,8 +402,9 @@ void _gst_update_object_memory_oop (OOP oop)
     /* Every allocation of a FloatD might cause a garbage
        collection! */
 #define SET_FIELD(x) \
-	data->x = floatd_new (_gst_mem.x); \
-	if (data != (gst_object_memory) OOP_TO_OBJ (oop)) continue;
+	floatOOP = floatd_new (_gst_mem.x); \
+	if (data != (gst_object_memory) OOP_TO_OBJ (oop)) continue; \
+	data->x = floatOOP;
 
     SET_FIELD (timeBetweenScavenges);
     SET_FIELD (timeBetweenGlobalGCs);
@@ -473,9 +484,8 @@ alloc_oop_table (size_t size)
   _gst_false_oop = &_gst_mem.ot[FALSE_OOP_INDEX];
 
   _gst_mem.num_free_oops = size;
-  _gst_mem.first_allocated_oop = _gst_mem.ot;
   _gst_mem.last_allocated_oop = _gst_mem.last_swept_oop = _gst_mem.ot - 1;
-  _gst_mem.highest_swept_oop = _gst_mem.ot;
+  _gst_mem.next_oop_to_sweep = _gst_mem.ot - 1;
 }
 
 mst_Boolean
@@ -624,8 +634,6 @@ _gst_make_oop_weak (OOP oop)
   weak_area_tree *node = NULL;
   rb_node_t **p = (rb_node_t **) &_gst_mem.weak_areas;
 
-  /* Weak OOPs must be fixed.  */
-  _gst_make_oop_fixed (oop);
   oop->flags |= F_WEAK;
   _gst_mem.numWeakOOPs++;
 
@@ -638,16 +646,10 @@ _gst_make_oop_weak (OOP oop)
       else if (oop > node->oop)
         p = &(*p)->rb_right;
       else
-        {
-	  node->base = (OOP *) oop->object;
-	  node->end = (OOP *) oop->object + TO_INT (oop->object->objSize);
-	  return;
-	}
+	return;
     }
 
   entry = (weak_area_tree *) xmalloc (sizeof (weak_area_tree));
-  entry->base = (OOP *) oop->object;
-  entry->end = (OOP *) oop->object + TO_INT (oop->object->objSize);
   entry->oop = oop;
   entry->rb.rb_parent = &node->rb;
   entry->rb.rb_left = entry->rb.rb_right = NULL;
@@ -772,7 +774,7 @@ _gst_alloc_obj (size_t size,
   newAllocPtr = _gst_mem.eden.allocPtr + BYTES_TO_SIZE (size);
 
   if UNCOMMON (size >= _gst_mem.big_object_threshold)
-    return _gst_alloc_old_obj (size, p_oop);
+    return alloc_old_obj (size, p_oop);
 
   if UNCOMMON (newAllocPtr >= _gst_mem.eden.maxPtr)
     {
@@ -789,14 +791,12 @@ _gst_alloc_obj (size_t size,
 }
 
 gst_object
-_gst_alloc_old_obj (size_t size,
-		    OOP *p_oop)
+alloc_old_obj (size_t size,
+	       OOP *p_oop)
 {
-  OOP *newAllocPtr;
   gst_object p_instance;
 
   size = ROUNDED_BYTES (size);
-  newAllocPtr = _gst_mem.eden.allocPtr + BYTES_TO_SIZE (size);
 
   /* If the object is big enough, we put it directly in oldspace.  */
   p_instance = (gst_object) _gst_mem_alloc (_gst_mem.old, size);
@@ -808,7 +808,7 @@ _gst_alloc_old_obj (size_t size,
   if COMMON (p_instance)
     goto ok;
 
-  _gst_compact (0);
+  compact (0);
   p_instance = (gst_object) _gst_mem_alloc (_gst_mem.old, size);
   if UNCOMMON (!p_instance)
     {
@@ -899,11 +899,30 @@ oldspace_before_freeing (heap_data *h, heap_block *blk, size_t sz)
   _gst_mem_protect ((PTR) blk, sz, PROT_READ | PROT_WRITE);
 }
 
-void
+heap_data *
 oldspace_nomemory (heap_data *h, size_t sz)
 {
+  heap_data **p_heap;
+
+  assert (h == _gst_mem.old || h == _gst_mem.fixed);
+  p_heap = (h == _gst_mem.old ? &_gst_mem.old : &_gst_mem.fixed);
+
   if (!_gst_gc_running)
     _gst_global_gc (sz);
+  else
+    {
+      /* Already garbage collecting, emergency growth just to satisfy
+	 tenuring necessities.  */
+      int grow_amount_to_satisfy_rate = h->heap_limit
+           * (100.0 + _gst_mem.space_grow_rate) / 100;
+      int grow_amount_to_satisfy_threshold = (sz + h->heap_total)
+	   * 100.0 /_gst_mem.grow_threshold_percent;
+
+      h->heap_limit = MAX (grow_amount_to_satisfy_rate,
+                           grow_amount_to_satisfy_threshold);
+    }
+
+  return *p_heap;
 }
 
 #ifndef NO_SIGSEGV_HANDLING
@@ -968,7 +987,7 @@ update_stats (unsigned long *last, double *between, double *duration)
 void
 _gst_grow_memory_to (size_t spaceSize)
 {
-  _gst_compact (spaceSize);
+  compact (spaceSize);
 }
 
 void
@@ -982,10 +1001,9 @@ grow_memory_no_compact (size_t new_heap_limit)
 }
 
 void
-_gst_compact (size_t new_heap_limit)
+compact (size_t new_heap_limit)
 {
   OOP oop;
-  grey_area_node *node, **next, *last;
   heap_data *new_heap = init_old_space (
     new_heap_limit ? new_heap_limit : _gst_mem.old->heap_limit);
 
@@ -999,30 +1017,12 @@ _gst_compact (size_t new_heap_limit)
     }
   else
     {
+      /* Do not copy garbage.  */
+      _gst_finish_incremental_gc ();
       _gst_mem.numCompactions++;
       update_stats (&stats.timeOfLastCompaction, NULL, NULL);
     }
 
-  /* Leave only pages from the loaded image in the grey table.  */
-  for (last = NULL, next = &_gst_mem.grey_pages.head; (node = *next); )
-    if (node->base >= (OOP *)_gst_mem.loaded_base
-        && node->base < _gst_mem.loaded_end)
-      {
-#ifdef MMAN_DEBUG_OUTPUT
-        printf ("  Remembered table entry left for loaded image: %p..%p\n",
-                node->base, node->base+node->n);
-#endif
-        last = node;
-        next = &(node->next);
-      }
-    else
-      {
-        _gst_mem.rememberedTableEntries--;
-        *next = node->next;
-        xfree (node);
-      }
-
-  _gst_mem.grey_pages.tail = last;
   _gst_fixup_object_pointers ();
 
   /* Now do the copying loop which will compact oldspace.  */
@@ -1056,8 +1056,7 @@ void
 _gst_global_compact ()
 {
   _gst_global_gc (0);
-  _gst_finish_incremental_gc ();
-  _gst_compact (0);
+  compact (0);
 }
 
 void
@@ -1115,12 +1114,26 @@ _gst_global_gc (int next_allocation)
     {
       old_limit = MAX (old_limit, _gst_mem.old->heap_total);
 
-      /* Check if it's time to compact the heap.  */
+      /* if memory is still low, go all the way on sweeping */
       if UNCOMMON ((next_allocation + _gst_mem.old->heap_total)
-	    * 100.0 / old_limit > _gst_mem.grow_threshold_percent)
+		    * 100.0 / old_limit > _gst_mem.grow_threshold_percent)
         {
-          s = "done, heap compacted";
-          _gst_compact (0);
+          int target_limit;
+          _gst_finish_incremental_gc ();
+
+	  /* Check if it's time to compact the heap. Compaction make the most 
+	     sense if there were lots of garbage. And the heap limit is shrunk 
+	     to avoid excessive garbage accumulation in the next round */
+	  target_limit = MAX(_gst_mem.eden.totalSize,
+			     ((next_allocation + _gst_mem.old->heap_total)
+			      * (100.0 + _gst_mem.space_grow_rate)
+			      / _gst_mem.grow_threshold_percent));
+	  if (target_limit < old_limit)
+            {
+              s = "done, heap compacted";
+              compact (0);
+              grow_memory_no_compact (target_limit);
+            }
         }
 
       /* Check if it's time to grow the heap.  */
@@ -1154,6 +1167,7 @@ _gst_global_gc (int next_allocation)
   if (!_gst_mem.old->heap_limit)
     _gst_mem.old->heap_limit = old_limit;
 
+  _gst_invalidate_croutine_cache ();
   mourn_objects ();
 }
 
@@ -1162,6 +1176,9 @@ _gst_scavenge (void)
 {
   int oldBytes, reclaimedBytes, tenuredBytes, reclaimedPercent;
 
+  /* Check if oldspace had to be grown in emergency.  */
+  size_t prev_heap_limit = _gst_mem.old->heap_limit;
+ 
   /* Force a GC as soon as possible if we're low on OOPs or memory.  */
   if UNCOMMON (_gst_mem.num_free_oops < LOW_WATER_OOP_THRESHOLD
      || _gst_mem.old->heap_total * 100.0 / _gst_mem.old->heap_limit >
@@ -1235,11 +1252,25 @@ _gst_scavenge (void)
     _gst_mem.factor * tenuredBytes +
     (1 - _gst_mem.factor) * _gst_mem.tenuredBytesPerScavenge;
 
+  _gst_invalidate_croutine_cache ();
   mourn_objects ();
+
+  /* If tenuring had to grow oldspace, do a global garbage collection
+     now.  */
+  if (_gst_mem.old->heap_limit > prev_heap_limit)
+    {
+      _gst_global_gc (0);
+      _gst_incremental_gc_step ();
+      return;
+    }
 }
 
 
-
+mst_Boolean
+incremental_gc_running ()
+{
+  return (_gst_mem.next_oop_to_sweep > _gst_mem.last_swept_oop);
+}
 
 void
 _gst_finish_incremental_gc ()
@@ -1248,37 +1279,48 @@ _gst_finish_incremental_gc ()
 
 #if defined (GC_DEBUG_OUTPUT)
   printf ("Completing sweep (%p...%p), validity flags %x\n", _gst_mem.last_swept_oop,
-          _gst_mem.highest_swept_oop, _gst_mem.live_flags);
+          _gst_mem.next_oop_to_sweep, _gst_mem.live_flags);
 #endif
 
-  PREFETCH_START (_gst_mem.highest_swept_oop, PREF_BACKWARDS | PREF_READ | PREF_NTA);
-  for (oop = _gst_mem.highest_swept_oop, firstOOP = _gst_mem.last_swept_oop;
-       --oop > firstOOP; oop->flags &= ~F_REACHABLE)
+  PREFETCH_START (_gst_mem.next_oop_to_sweep, PREF_BACKWARDS | PREF_READ | PREF_NTA);
+  for (oop = _gst_mem.next_oop_to_sweep, firstOOP = _gst_mem.last_swept_oop;
+       oop > firstOOP; oop--)
     {
       PREFETCH_LOOP (oop, PREF_BACKWARDS | PREF_READ | PREF_NTA);
-      if (!IS_OOP_VALID_GC (oop))
+      if (IS_OOP_VALID_GC (oop))
+	{
+	  maybe_release_xlat (oop);
+	  oop->flags &= ~F_REACHABLE;
+	}
+      else
         {
           _gst_sweep_oop (oop);
 	  _gst_mem.num_free_oops++;
-          _gst_mem.highest_swept_oop = oop;
           if (oop == _gst_mem.last_allocated_oop)
             _gst_mem.last_allocated_oop--;
         }
     }
 
-  finished_incremental_gc ();
+  _gst_mem.next_oop_to_sweep = oop;
+  _gst_finished_incremental_gc ();
 }
 
 void
-finished_incremental_gc (void)
+_gst_finished_incremental_gc (void)
 {
+  if (_gst_mem.live_flags & F_OLD)
+    return;
+
   _gst_mem.live_flags &= ~F_REACHABLE;
   _gst_mem.live_flags |= F_OLD;
 
-  _gst_mem.reclaimedBytesPerGlobalGC =
-    _gst_mem.factor * stats.reclaimedOldSpaceBytesSinceLastGlobalGC +
-    (1 - _gst_mem.factor) * _gst_mem.reclaimedBytesPerScavenge;
-
+  if (stats.reclaimedOldSpaceBytesSinceLastGlobalGC) 
+    {
+      _gst_mem.reclaimedBytesPerGlobalGC =
+        _gst_mem.factor * stats.reclaimedOldSpaceBytesSinceLastGlobalGC +
+        (1 - _gst_mem.factor) * _gst_mem.reclaimedBytesPerGlobalGC;
+      stats.reclaimedOldSpaceBytesSinceLastGlobalGC = 0;
+    }
 #ifdef ENABLE_JIT_TRANSLATION
   /* Go and really free the blocks associated to garbage collected
      native code.  */
@@ -1286,33 +1328,41 @@ finished_incremental_gc (void)
 #endif
 }
 
-void
+mst_Boolean
 _gst_incremental_gc_step ()
 {
   OOP oop, firstOOP;
   int i;
 
-  for (i = 0, oop = _gst_mem.highest_swept_oop,
-       firstOOP = _gst_mem.last_swept_oop;
-       i <= INCREMENTAL_SWEEP_STEP && --oop > firstOOP;
-       oop->flags &= ~F_REACHABLE)
-    {
-      if (--oop > firstOOP)
-        {
-	  finished_incremental_gc ();
-	  break;
-	}
+  if (!incremental_gc_running ())
+    return true;
 
-      if (!IS_OOP_VALID_GC (oop))
+  i = 0;
+  firstOOP = _gst_mem.last_swept_oop;
+  for (oop = _gst_mem.next_oop_to_sweep; oop > firstOOP; oop--)
+    {
+      if (IS_OOP_VALID_GC (oop))
+	{
+	  maybe_release_xlat (oop);
+	  oop->flags &= ~F_REACHABLE;
+	}
+      else
         {
-          i++;
           _gst_sweep_oop (oop);
   	  _gst_mem.num_free_oops++;
-          _gst_mem.highest_swept_oop = oop;
           if (oop == _gst_mem.last_allocated_oop)
             _gst_mem.last_allocated_oop--;
+	  if (++i == INCREMENTAL_SWEEP_STEP)
+	    {
+	      _gst_mem.next_oop_to_sweep = oop - 1;
+	      return false;
+	    }
         }
     }
+
+  _gst_mem.next_oop_to_sweep = oop;
+  _gst_finished_incremental_gc ();
+  return true;
 }
 
 void
@@ -1344,21 +1394,21 @@ reset_incremental_gc (OOP firstOOP)
       ;
 #endif
 
-  _gst_mem.first_allocated_oop = oop;
+  /* Initialize these here so that IS_OOP_VALID works correctly.  */
+  _gst_mem.next_oop_to_sweep = _gst_mem.last_allocated_oop;
+  _gst_mem.last_swept_oop = oop - 1;
 
 #ifdef NO_INCREMENTAL_GC
-  _gst_mem.highest_swept_oop = _gst_mem.last_allocated_oop + 1;
   _gst_finish_incremental_gc ();
 #else
   /* Skip high OOPs that are unallocated.  */
-  for (oop = _gst_mem.last_allocated_oop; !IS_OOP_VALID_GC (oop); oop--)
+  for (oop = _gst_mem.last_allocated_oop; !IS_OOP_VALID (oop); oop--)
     _gst_sweep_oop (oop);
 
   _gst_mem.last_allocated_oop = oop;
-  _gst_mem.highest_swept_oop = oop + 1;
+  _gst_mem.next_oop_to_sweep = oop;
 #endif
 
-  _gst_mem.last_swept_oop = _gst_mem.first_allocated_oop - 1;
   _gst_mem.num_free_oops = _gst_mem.ot_size -
     (_gst_mem.last_allocated_oop - _gst_mem.ot);
 
@@ -1369,10 +1419,10 @@ reset_incremental_gc (OOP firstOOP)
       * (100 + _gst_mem.space_grow_rate) / 100);
 
 #if defined (GC_DEBUG_OUTPUT)
-  printf ("Found unallocated at OOP %p, last allocated OOP %p\n"
-          "Highest OOP swept top to bottom %p, lowest swept bottom to top %p\n",
-	  _gst_mem.first_allocated_oop, _gst_mem.last_allocated_oop,
-	  _gst_mem.highest_swept_oop, _gst_mem.last_swept_oop);
+  printf ("Last allocated OOP %p\n"
+          "Next OOP swept top to bottom %p, highest swept bottom to top %p\n",
+	  _gst_mem.last_allocated_oop,
+	  _gst_mem.next_oop_to_sweep, _gst_mem.last_swept_oop);
 #endif
 }
 
@@ -1380,8 +1430,6 @@ reset_incremental_gc (OOP firstOOP)
 void
 _gst_sweep_oop (OOP oop)
 {
-  _gst_mem.last_swept_oop = oop;
-
   if (IS_OOP_FREE (oop))
     return;
 
@@ -1440,7 +1488,18 @@ mourn_objects (void)
       /* Copy the buffer into an Array */
       array = new_instance_with (_gst_array_class, size, &processor->gcArray);
       _gst_copy_buffer (array->data);
-      _gst_async_signal (processor->gcSemaphore);
+      if (!IS_NIL (processor->gcSemaphore))
+        {
+          static async_queue_entry e;
+          e.func = _gst_do_async_signal;
+          e.data = processor->gcSemaphore;
+          _gst_async_call_internal (&e);
+        }
+      else
+	{
+	  _gst_errorf ("Running finalizers before initialization.");
+	  abort ();
+	}
     }
 }
 
@@ -1494,8 +1553,7 @@ queue_put (surv_space *q, OOP *src, int n)
           printf ("Wrap: survivor space ends at %p, needed %p\n", q->maxPtr, newAlloc);
 #endif
           q->topPtr = q->allocPtr;
-          result = q->allocPtr = q->minPtr;
-          newAlloc = q->allocPtr + n;
+          q->allocPtr = q->minPtr;
 	  continue;
         }
 
@@ -1643,12 +1701,15 @@ check_weak_refs ()
     {
       weak_area_tree *area = (weak_area_tree *) node;
       mst_Boolean mourn = false;
-      OOP *field;
+      OOP *field, oop;
+      int n;
 
-      if (!IS_OOP_VALID_GC (area->oop))
+      oop = area->oop;
+      if (!IS_OOP_VALID_GC (oop))
 	continue;
 
-      for (field = area->base; field < area->end; field++)
+      for (field = (OOP *) oop->object + OBJ_HEADER_SIZE_WORDS,
+	   n = NUM_OOPS (oop->object); n--; field++)
         {
 	  OOP oop = *field;
           if (IS_INT (oop))
@@ -1754,7 +1815,12 @@ scan_grey_pages ()
   for (last = NULL, next = &_gst_mem.grey_pages.head; (node = *next); )
     {
 #if defined(GC_DEBUG_OUTPUT) || defined(MMAN_DEBUG_OUTPUT)
-      printf ("Scanning grey page %p...%p\n", node->base, node->base + node->n);
+      printf ("Scanning grey page %p...%p ", node->base, node->base + node->n);
+#if defined(GC_DEBUG_OUTPUT)
+      putchar ('\n');
+#else
+      fflush (stdout);
+#endif
 #endif
 
       PREFETCH_START (node->base, PREF_READ | PREF_NTA);
@@ -1791,6 +1857,9 @@ scan_grey_pages ()
       else
 #endif
 	{
+#if defined (MMAN_DEBUG_OUTPUT)
+	  printf ("Found %d pointers\n", n);
+#endif
 	  last = node;
 	  next = &(node->next);
 	}
@@ -1858,15 +1927,10 @@ int
 scanned_fields_in (gst_object object,
 		  int flags)
 {
-  OOP objClass = object->objClass;
-
   if COMMON (!(flags & (F_WEAK | F_CONTEXT)))
     {
       int size = NUM_OOPS (object);
-      if COMMON (size)
-	return object->data + size - &object->objClass;
-      else
-	return UNCOMMON (!IS_OOP_COPIED (objClass));
+      return object->data + size - &object->objClass;
     }
 
   if COMMON (flags & F_CONTEXT)
@@ -1877,14 +1941,9 @@ scanned_fields_in (gst_object object,
       methodSP = TO_INT (ctx->spOffset);
       return ctx->contextStack + methodSP + 1 - &ctx->objClass;
     }
-  else
-    {
-      /* In general, there will be many instances of a class,
-	 but only the first time will it need to be copied;
-         moreover, classes are often old.  So I'm
-	 marking this as uncommon.  */
-      return UNCOMMON (!IS_OOP_COPIED (objClass));
-    }
+
+  /* Weak object, only mark the class.  */
+  return 1;
 }
 
 void
@@ -2093,134 +2152,167 @@ mark_ephemeron_oops (void)
   _gst_add_buf_data (base, (char *) pDeadOOP - (char *) base);
 }
 
-#define TAIL_MARK_OOP(newOOP) BEGIN_MACRO { \
+#define TAIL_MARK_OOP(newOOP) do { \
   PREFETCH_ADDR ((newOOP)->object, PREF_READ | PREF_NTA); \
   oop = (newOOP); \
-  continue;		/* tail recurse!!! */ \
-} END_MACRO
+  goto markOne;		/* tail recurse!!! */ \
+} while(0)
 
-#define TAIL_MARK_OOPRANGE(firstOOP, oopAtEnd) BEGIN_MACRO { \
+#define TAIL_MARK_OOPRANGE(firstOOP, oopAtEnd) do { \
   PREFETCH_START (firstOOP, PREF_READ | PREF_NTA); \
   curOOP = (OOP *)(firstOOP); \
   atEndOOP = (OOP *)(oopAtEnd); \
   oop = NULL; \
-  continue; \
-} END_MACRO
+  goto markRange; \
+} while(0)
 
 void
-_gst_mark_an_oop_internal (OOP oop,
-			   OOP * curOOP,
-			   OOP * atEndOOP)
+_gst_mark_an_oop_internal (OOP oop)
 {
-  for (;;)
-    {
-      if (!oop)
-	{			/* in the loop! */
-#if defined (GC_DEBUGGING)
-	  gst_object obj = (gst_object) (curOOP - 1);	/* for debugging */
-#endif
-	iterationLoop:
-	  /* in a loop, do next iteration */
-	  oop = *curOOP;
-	  PREFETCH_LOOP (curOOP, PREF_READ | PREF_NTA);
-	  curOOP++;
-	  if (IS_OOP (oop))
-	    {
-#if defined (GC_DEBUGGING)
-	      if UNCOMMON (!IS_OOP_ADDR (oop))
-		{
-		  printf
-		    ("Error! Invalid OOP %p was found inside %p!\n",
-		     oop, obj);
-		  abort ();
-		}
-	      else
-#endif
-	      if (!IS_OOP_MARKED (oop))
-		{
-		  PREFETCH_START (oop->object, PREF_READ | PREF_NTA);
-		  if COMMON (curOOP < atEndOOP)
-		    {
-		      _gst_mark_an_oop_internal (oop, NULL, NULL);
-		      goto iterationLoop;
-		    }
-		  else
-		    /* On the last object in the set, reuse the
-		       current invocation. oop is valid, so we go to
-		       the single-object case */
-		    continue;
-		}
-	    }
-	  /* We reach this point if the object isn't to be marked.  The 
-	     code above contains a continue to tail recurse, so we
-	     cannot put the loop in a do...while and a goto is
-	     necessary here.  Speed is a requirement, so I'm doing it.  */
-	  if (curOOP < atEndOOP)
-	    goto iterationLoop;
-	}
-      else
-	{			/* just starting with this oop */
-	  OOP objClass;
-	  gst_object object;
-	  uintptr_t size;
+  OOP *curOOP, *atEndOOP;
+  struct mark_queue *markQueue = _gst_mem.markQueue;
+  struct mark_queue *lastMarkQueue = _gst_mem.lastMarkQueue;
+  struct mark_queue *currentMarkQueue = markQueue;
+  goto markOne;
+
+ markRange:
+  {
+    OOP firstOOP = NULL;
+
+    /* The first unmarked OOP is used for tail recursion.  */
+    while (curOOP < atEndOOP)
+      {
+        oop = *curOOP++;
+        if (IS_OOP (oop) && !IS_OOP_MARKED (oop))
+          {
+            oop->flags |= F_REACHABLE;
+            firstOOP = oop;
+            break;
+          }
+      }
+
+    /* The second unmarked OOP is the first that is placed on the mark
+       queue.  TODO: split REACHABLE and VISITED flags.  An object is
+       marked REACHABLE here, and REACHABLE|VISITED in the markOne label.
+       At the end of GC, all REACHABLE objects are also VISITED.
+       The above loop should seek an object that is not VISITED so
+       that it can be marked.  For the loop below, however, REACHABLE
+       objects are known to be somewhere else on the mark stack, and can
+       be skipped.
+
+       Skipping objects after the first unmarked OOP is still useful,
+       because it keeps the stack size a bit lower in the relatively common
+       case of many integer or nil instance variables.  */
+    while (curOOP < atEndOOP)
+      {
+        oop = *curOOP;
+
+        if (IS_OOP (oop) && !IS_OOP_MARKED (oop))
+          {
+            if (currentMarkQueue == lastMarkQueue)
+              {
+                const size_t size = lastMarkQueue - markQueue;
+
+                _gst_mem.markQueue = (struct mark_queue *)
+                  xrealloc (_gst_mem.markQueue, 2 * size * sizeof (struct mark_queue));
+                _gst_mem.lastMarkQueue = &_gst_mem.markQueue[2 * size];
+
+		markQueue = _gst_mem.markQueue;
+		lastMarkQueue = _gst_mem.lastMarkQueue;
+                currentMarkQueue = &_gst_mem.markQueue[size];
+              }
+            currentMarkQueue->firstOOP = curOOP;
+            currentMarkQueue->endOOP = atEndOOP;
+	    currentMarkQueue++;
+            break;
+          }
+
+        curOOP++;
+      }
+
+    if (!firstOOP)
+      goto pop;
+
+    TAIL_MARK_OOP (firstOOP);
+  }
+
+ markOne:
+  {
+    OOP objClass;
+    gst_object object;
+    uintptr_t size;
 
 #if defined (GC_DEBUGGING)
-	  if UNCOMMON (IS_OOP_FREE (oop))
-	    {
-	      printf ("Error! Free OOP %p is being marked!\n", oop);
-	      abort ();
-	      break;
-	    }
+    if UNCOMMON (IS_OOP_FREE (oop))
+      {
+        printf ("Error! Free OOP %p is being marked!\n", oop);
+        abort ();
+        return;
+      }
 #endif
 
 #if defined(GC_DEBUG_OUTPUT)
-	  printf (">Mark ");
-	  _gst_display_oop (oop);
+    printf (">Mark ");
+    _gst_display_oop (oop);
 #endif
 
-	  /* see if the object has pointers, set up to copy them if so. 
-	   */
-	  oop->flags |= F_REACHABLE;
-	  object = OOP_TO_OBJ (oop);
-	  objClass = object->objClass;
-	  if UNCOMMON (oop->flags & F_CONTEXT)
-	    {
-	      gst_method_context ctx;
-	      intptr_t methodSP;
-	      ctx = (gst_method_context) object;
-	      methodSP = TO_INT (ctx->spOffset);
-	      /* printf("setting up for loop on context %x, sp = %d\n", 
-	         ctx, methodSP); */
-	      TAIL_MARK_OOPRANGE (&ctx->objClass,
-				  ctx->contextStack + methodSP + 1);
+    /* see if the object has pointers, set up to copy them if so. 
+    */
+    oop->flags |= F_REACHABLE;
+    object = OOP_TO_OBJ (oop);
+    objClass = object->objClass;
+    if UNCOMMON (oop->flags & F_CONTEXT)
+      {
+        gst_method_context ctx;
+        intptr_t methodSP;
+        ctx = (gst_method_context) object;
+        methodSP = TO_INT (ctx->spOffset);
+        /* printf("setting up for loop on context %x, sp = %d\n", 
+           ctx, methodSP); */
+        TAIL_MARK_OOPRANGE (&ctx->objClass,
+                            ctx->contextStack + methodSP + 1);
 
-	    }
-	  else if UNCOMMON (oop->flags & (F_EPHEMERON | F_WEAK))
-	    {
-	      if (oop->flags & F_EPHEMERON)
-	        _gst_add_buf_pointer (oop);
+      }
+    else if UNCOMMON (oop->flags & (F_EPHEMERON | F_WEAK))
+      {
+        if (oop->flags & F_EPHEMERON)
+          _gst_add_buf_pointer (oop);
 
-	      /* In general, there will be many instances of a class,
-		 but only the first time will it be unmarked.  So I'm
-		 marking this as uncommon.  */
-	      if UNCOMMON (!IS_OOP_MARKED (objClass))
-		TAIL_MARK_OOP (objClass);
-	    }
-	  else
-	    {
-	      size = NUM_OOPS (object);
-	      if COMMON (size)
-		TAIL_MARK_OOPRANGE (&object->objClass,
-				    object->data + size);
+        /* In general, there will be many instances of a class,
+           but only the first time will it be unmarked.  So I'm
+           marking this as uncommon.  */
+        if UNCOMMON (!IS_OOP_MARKED (objClass))
+          TAIL_MARK_OOP (objClass);
+      }
+    else
+      {
+        size = NUM_OOPS (object);
+        if COMMON (size)
+          TAIL_MARK_OOPRANGE (&object->objClass,
+                              object->data + size);
 
-	      else if UNCOMMON (!IS_OOP_MARKED (objClass))
-		TAIL_MARK_OOP (objClass);
-	    }
-	}
-      /* This point is reached if and only if nothing has to be marked
-         anymore in the current iteration. So exit.  */
-      break;
-    }				/* for(;;) */
+        else if UNCOMMON (!IS_OOP_MARKED (objClass))
+          TAIL_MARK_OOP (objClass);
+      }
+  }
+
+ pop:
+  {
+    if (currentMarkQueue > markQueue)
+      {
+        currentMarkQueue--;
+	TAIL_MARK_OOPRANGE (currentMarkQueue->firstOOP,
+			    currentMarkQueue->endOOP);
+      }
+  }
+}
+
+void
+_gst_mark_oop_range (OOP *curOOP, OOP *atEndOOP)
+{
+  OOP *pOOP;
+  for (pOOP = curOOP; pOOP < atEndOOP; pOOP++)
+    MAYBE_MARK_OOP (*pOOP);
 }
 
 
@@ -2241,14 +2333,11 @@ _gst_inc_init_registry (void)
 void
 _gst_inc_grow_registry (void)
 {
-  OOP *oldBase;
   unsigned oldPtrOffset;
   unsigned oldRegistrySize, newRegistrySize;
 
-  oldBase = _gst_mem.inc_base;
   oldPtrOffset = _gst_mem.inc_ptr - _gst_mem.inc_base;
   oldRegistrySize = _gst_mem.inc_end - _gst_mem.inc_base;
-
   newRegistrySize = oldRegistrySize + INCUBATOR_CHUNK_SIZE;
 
   _gst_mem.inc_base =
