@@ -1,7 +1,7 @@
 /*  -*- Mode: C -*-  */
 
 /* printf.c --- printf clone for argv arrays
- * Copyright (C) 1998, 1999, 2000, 2002 Gary V. Vaughan
+ * Copyright (C) 1998, 1999, 2000, 2002, 2009 Gary V. Vaughan
  * Originally by Gary V. Vaughan, 1998
  * This file is part of Snprintfv
  *
@@ -43,7 +43,6 @@
 #include "filament.h"
 #include "stream.h"
 #include "mem.h"
-#include "dl.h"
 
 #define EOS			'\0'
 #define SNV_CHAR_SPEC		'%'
@@ -95,8 +94,6 @@ spec_init (void)
 	  index++;
 	}
 
-      snv_load_all_modules ();
-
       is_init = TRUE;
     }
 }
